VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cFrame"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Modul        : cFrame
' Beschreibung : VB-Frame Control Replacement
'                Mit transparentem Hintergrund
'
' Erstellt von : Christoph von Wittich
' Erstellt am  : Donnerstag, 15.05.2003
'
' Copyright (C) 2003 bei Christoph von Wittich, alle Rechte vorbehalten.
' Kontakt: Christoph@ActiveVB.de

Option Explicit

Private Const LOGPIXELSY As Long = 90

Private Const BS_GROUPBOX = &H7&
Private Const WS_VISIBLE = &H10000000
Private Const WS_CLIPCHILDREN = &H2000000
Private Const WS_CLIPSIBLINGS = &H4000000
Private Const WS_CHILD = &H40000000

Private Const WS_EX_TRANSPARENT = &H20&
Private Const WS_EX_CONTROLPARENT = &H10000

Private Const SW_SHOWNA As Long = 8
Private Const SW_HIDE As Long = 0

Private Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Private Type POINTAPI
    x As Long
    y As Long
End Type

'USER32
Private Declare Function CreateWindowEx Lib "user32" Alias "CreateWindowExA" (ByVal dwExStyle As Long, ByVal lpClassName As String, ByVal lpWindowName As String, ByVal dwStyle As Long, ByVal x As Long, ByVal y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hWndParent As Long, ByVal hMenu As Long, ByVal hInstance As Long, lpParam As Any) As Long
Private Declare Function SendMessage Lib "user32.dll" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function ShowWindow Lib "user32.dll" (ByVal hwnd As Long, ByVal nCmdShow As Long) As Long
Private Declare Function EnableWindow Lib "user32.dll" (ByVal hwnd As Long, ByVal fEnable As Long) As Long
Private Declare Function IsWindowEnabled Lib "user32.dll" (ByVal hwnd As Long) As Long
Private Declare Function GetWindowRect Lib "user32.dll" (ByVal hwnd As Long, lpRect As RECT) As Long
Private Declare Function GetParent Lib "user32.dll" (ByVal hwnd As Long) As Long
Private Declare Function ScreenToClient Lib "user32.dll" (ByVal hwnd As Long, lpPoint As POINTAPI) As Long
Private Declare Function MoveWindow Lib "user32.dll" (ByVal hwnd As Long, ByVal x As Long, ByVal y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal bRepaint As Long) As Long
Private Declare Function GetDC Lib "user32.dll" (ByVal hwnd As Long) As Long
Private Declare Function GetActiveWindow Lib "user32.dll" () As Long

'GDI32
Private Declare Function DeleteObject Lib "gdi32.dll" (ByVal hObject As Long) As Long
Private Declare Function GetDeviceCaps Lib "gdi32.dll" (ByVal hdc As Long, ByVal nIndex As Long) As Long
Private Declare Function CreateFont Lib "gdi32.dll" Alias "CreateFontA" (ByVal H As Long, ByVal W As Long, ByVal E As Long, ByVal O As Long, ByVal W As Long, ByVal I As Long, ByVal u As Long, ByVal S As Long, ByVal C As Long, ByVal OP As Long, ByVal CP As Long, ByVal Q As Long, ByVal PAF As Long, ByVal F As String) As Long

'KERNEL32
Private Declare Function MulDiv Lib "kernel32.dll" (ByVal nNumber As Long, ByVal nNumerator As Long, ByVal nDenominator As Long) As Long

Private hFont As Long
Private sFont As String
Private lHandle As Long
Private lFontSize As Long
Private bVisible As Boolean
Private mTwipsPerPixelX As Long
Private mTwipsPerPixelY As Long

Public Tag As String

Public Sub Create(mParent As Long, x As Long, y As Long, Width As Long, _
Height As Long, Optional mCaption As String)
    mTwipsPerPixelX = Screen.TwipsPerPixelX
    mTwipsPerPixelY = Screen.TwipsPerPixelY
    lHandle = CreateWindowEx(WS_EX_TRANSPARENT Or WS_EX_CONTROLPARENT, _
        "Button", mCaption, WS_CLIPCHILDREN Or WS_CLIPSIBLINGS Or WS_CHILD Or _
        WS_VISIBLE Or BS_GROUPBOX, x, y, Width, Height, mParent, 0, 0, 0)
End Sub

Public Property Set Font(mFont As StdFont)
    sFont = mFont
    DeleteObject hFont
    hFont = CreateFont(-MulDiv(mFont.Size, _
        GetDeviceCaps(GetDC(GetActiveWindow), LOGPIXELSY), 72), 0, 0, 0, 0, 0, 0, 0, _
        0, 0, 0, 0, 0, mFont.Name)
    SendMessage lHandle, &H30, hFont, 1
End Property

'-----------------------------------------------------------
' PROPERTY LET: Visible
'
' Legt fest, ob das Control sichtbar oder unsichtbar ist
'-----------------------------------------------------------
Public Property Let Visible(mVisible As Boolean)
    If mVisible Then
        ShowWindow lHandle, SW_SHOWNA
    Else
        ShowWindow lHandle, SW_HIDE
    End If
    bVisible = mVisible
End Property

'-----------------------------------------------------------
' PROPERTY GET: Visible
'
' Gibt zurck, ob das Control sichtbar oder unsichtbar ist
'-----------------------------------------------------------
Public Property Get Visible() As Boolean
    Visible = bVisible
End Property

'-----------------------------------------------------------
' PROPERTY LET: Enabled
'
' Legt fest, ob das Control aktiviert oder deaktiviert ist
'-----------------------------------------------------------
Public Property Let Enabled(mEnabled As Boolean)
    EnableWindow lHandle, Abs(mEnabled)
End Property

'-----------------------------------------------------------
' PROPERTY GET: Enabled
'
' Gibt zurck, ob das Control aktiviert oder deaktiviert ist
'-----------------------------------------------------------
Public Property Get Enabled() As Boolean
    Enabled = IsWindowEnabled(lHandle)
End Property

'-----------------------------------------------------------
' SUB: Move
'
' Verschiebt das Control an die angegebenen Koordinaten
'-----------------------------------------------------------
Public Sub Move(Left As Single, Optional Top, Optional Width, Optional Height)
    Dim r As RECT, P As POINTAPI
    Dim mWidth As Single, mLeft As Single
    Dim mHeight As Single, mTop As Single
    GetWindowRect lHandle, r
    P.x = r.Left
    P.y = r.Top
    ScreenToClient GetParent(lHandle), P
    If IsMissing(Width) Then mWidth = r.Right - r.Left Else mWidth = Width / mTwipsPerPixelX
    If IsMissing(Top) Then mTop = P.y Else mTop = Top / mTwipsPerPixelY
    If IsMissing(Left) Then mLeft = P.x Else mLeft = Left / mTwipsPerPixelX
    If IsMissing(Height) Then mHeight = r.Bottom - r.Top Else mHeight = Height / mTwipsPerPixelY
    MoveWindow lHandle, mLeft, mTop, mWidth, mHeight, 1
End Sub

'-----------------------------------------------------------
' PROPERTY GET: Left
'
' Gibt den Abstand zum linken Rand des Parent-Windows zurck
'-----------------------------------------------------------
Public Property Get Left() As Single
    Dim r As RECT, P As POINTAPI
    GetWindowRect lHandle, r
    P.x = r.Left
    P.y = r.Top
    ScreenToClient GetParent(lHandle), P
    Left = P.x * mTwipsPerPixelX
End Property

'-----------------------------------------------------------
' PROPERTY GET: Top
'
' Gibt den Abstand zum oberen Rand des Parent-Windows zurck
'-----------------------------------------------------------
Public Property Get Top() As Single
    Dim r As RECT, P As POINTAPI
    GetWindowRect lHandle, r
    P.x = r.Left
    P.y = r.Top
    ScreenToClient GetParent(lHandle), P
    Top = P.y * mTwipsPerPixelY
End Property

'-----------------------------------------------------------
' PROPERTY GET: Height
'
' Gibt die Hhe zurck
'-----------------------------------------------------------
Public Property Get Height() As Single
    Dim r As RECT
    GetWindowRect lHandle, r
    Height = (r.Bottom - r.Top) * mTwipsPerPixelY
End Property

'-----------------------------------------------------------
' PROPERTY GET: Width
'
' Gibt die Breite zurck
'-----------------------------------------------------------
Public Property Get Width() As Single
    Dim r As RECT
    GetWindowRect lHandle, r
    Width = (r.Right - r.Left) * mTwipsPerPixelX
End Property

'-----------------------------------------------------------
' PROPERTY GET: hDC
'
' Gibt den Gertekontext zurck
'-----------------------------------------------------------
Public Property Get hdc() As Long
    hdc = GetDC(lHandle)
End Property

'-----------------------------------------------------------
' PROPERTY LET: Left
'
' Setzt den Abstand vom linken Rand des Parent-Window
'-----------------------------------------------------------
Public Property Let Left(mLeft As Single)
    Move mLeft
End Property

'-----------------------------------------------------------
' PROPERTY LET: Top
'
' Setzt den Abstand vom oberen Rand des Parent-Window
'-----------------------------------------------------------
Public Property Let Top(mTop As Single)
    Move Left, mTop
End Property

'-----------------------------------------------------------
' PROPERTY LET: Width
'
' Legt die Breite fest
'-----------------------------------------------------------
Public Property Let Width(mWidth As Single)
    Move Left, Top, mWidth, Height
End Property

'-----------------------------------------------------------
' PROPERTY LET: Height
'
' Legt die Hhe fest
'-----------------------------------------------------------
Public Property Let Height(mHeight As Single)
    Move Left, Top, Width, mHeight
End Property

